@@ -43,6 +43,7 @@
 #include "gm-text-anchored-tag.h"
 #include "gm-text-smiley.h"
 #include "gm-text-extlink.h"
+#include "gm-smiley-chooser-button.h"
 
 #include "gm-smileys.h"
 #include "toolbox/toolbox.h"
@@ -142,11 +143,9 @@ static gboolean on_extlink_tag_event (GtkTextTag* tag,
 				      GtkTextIter* iter,
 				      gpointer data);
 
-static void on_smiley_activated (GtkMenuItem *item,
-				 gpointer data);
-
-static void on_smiley_clicked (GtkButton* button,
-			       gpointer data);
+static void on_smiley_selected (GmSmileyChooserButton*,
+				gpointer,
+				gpointer);
 
 static void on_font_changed (GtkButton* button,
                              gpointer data);
@@ -157,6 +156,13 @@ static gboolean message_activated_cb (GtkWidget *w,
 
 static void on_chat_removed (ChatArea* self);
 
+static void on_chat_area_grab_focus (GtkWidget*,
+				     gpointer);
+
+static gboolean on_chat_area_focus (GtkWidget*,
+				    GtkDirectionType,
+				    gpointer);
+
 /* implementation of internal api */
 
 static void
@@ -238,7 +244,7 @@ chat_area_add_message (ChatArea* self,
   GtkTextBuffer* buffer = NULL;
   GtkTextIter iter;
 
-  str = g_strdup_printf ("<i>%s %s</i> %s\n", from, _("says:"), txt);
+  str = g_strdup_printf ("<b><i>%s %s</i></b>\n%s\n", from, _("says:"), txt);
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->priv->text_view));
   gtk_text_buffer_get_end_iter (buffer, &iter);
   gm_text_buffer_enhancer_insert_text (self->priv->enhancer, &iter,
@@ -434,8 +440,9 @@ on_extlink_tag_event (GtkTextTag* tag,
 }
 
 static void
-on_smiley_activated (GtkMenuItem *item,
-		     gpointer data)
+on_smiley_selected (G_GNUC_UNUSED GmSmileyChooserButton* smiley_chooser_button,
+		    gpointer characters,
+		    gpointer data)
 {
   const gchar* text = NULL;
   ChatArea* self = NULL;
@@ -444,8 +451,7 @@ on_smiley_activated (GtkMenuItem *item,
 
   self = (ChatArea*)data;
 
-  /* FIXME: that will break when gtk+ will change... */
-  text = gtk_label_get_text (GTK_LABEL(GTK_BIN(GTK_MENU_ITEM (item))->child));
+  text = g_strdup ((gchar*) characters);
 
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->priv->message));
   gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_insert (buffer));
@@ -453,67 +459,81 @@ on_smiley_activated (GtkMenuItem *item,
   gtk_widget_grab_focus (self->priv->message);
 }
 
-static void
-on_smiley_clicked (G_GNUC_UNUSED GtkButton* button,
-		   gpointer data)
-{
-  ChatArea* self = NULL;
-
-  self = (ChatArea*)data;
-
-  gtk_menu_popup (GTK_MENU (self->priv->smiley_menu),
-		  NULL, NULL, NULL, NULL, 0, 0);
-}
-
 static void
 on_font_changed (GtkButton* button,
                  gpointer data)
 {
   ChatArea* self = NULL;
   GtkTextBuffer *buffer = NULL;
-  GtkTextMark *mark = NULL;
+  GtkTextMark* mark = NULL;
+  GtkTextMark* mark_at_insert = NULL;
+  GtkTextMark* mark_at_bound = NULL;
   GtkTextIter start;
   GtkTextIter end;
+  GtkTextIter iter_sav_insert;
+  GtkTextIter iter_sav_bound;
 
-  const char* opening_tag = NULL;
-  const char* closing_tag = NULL;
-  const char* tags = NULL;
+  const gchar* opening_tag = NULL;
+  const gchar* closing_tag = NULL;
+  gchar* tags = NULL;
 
   self = (ChatArea*)data;
 
-  if (!strcmp (gtk_button_get_label (GTK_BUTTON (button)), GTK_STOCK_BOLD)) {
-
-    opening_tag = "<b>";
-    closing_tag = "</b>";
-    tags = "<b></b>";
-  }
-  else if (!strcmp (gtk_button_get_label (GTK_BUTTON (button)), GTK_STOCK_ITALIC)) {
-
-    opening_tag = "<i>";
-    closing_tag = "</i>";
-    tags = "<i></i>";
-  }
-  else if (!strcmp (gtk_button_get_label (GTK_BUTTON (button)), GTK_STOCK_UNDERLINE)) {
-
-    opening_tag = "<u>";
-    closing_tag = "</u>";
-    tags = "<u></u>";
-  }
-
+  /* FIXME it's somehow dangerous to not check if we have these associations
+   * set for the button object?
+   * -Jan */
+  opening_tag = (const gchar*) g_object_get_data (G_OBJECT (button),
+						  "gm_open_tag");
+  closing_tag = (const gchar*) g_object_get_data (G_OBJECT (button),
+						  "gm_close_tag");
+  tags = g_strdup_printf ("%s%s", opening_tag, closing_tag);
 
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->priv->message));
   if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end)) {
 
-    mark = gtk_text_buffer_create_mark (buffer, "end", &end, false);
+    /* "save" the current selection by inserting marks with different gravity
+     * depending on the order of insert/bound in the buffer */
+    gtk_text_buffer_get_iter_at_mark (buffer, &iter_sav_insert,
+				      gtk_text_buffer_get_insert (buffer));
+    gtk_text_buffer_get_iter_at_mark (buffer, &iter_sav_bound,
+				      gtk_text_buffer_get_selection_bound (buffer));
+    if (gtk_text_iter_compare (&iter_sav_bound, &iter_sav_insert) < 0) {
+      /* selection is left-right */
+      mark_at_bound = gtk_text_buffer_create_mark (buffer, NULL,
+						   &iter_sav_bound, FALSE);
+      mark_at_insert = gtk_text_buffer_create_mark (buffer, NULL,
+						    &iter_sav_insert, TRUE);
+      mark = mark_at_insert;
+    } else {
+      /* selection is right-left */
+      mark_at_bound = gtk_text_buffer_create_mark (buffer, NULL,
+						   &iter_sav_bound, TRUE);
+      mark_at_insert = gtk_text_buffer_create_mark (buffer, NULL,
+						    &iter_sav_insert, FALSE);
+      mark = mark_at_bound;
+    }
+
+    /* actually insert the tags */
     gtk_text_buffer_insert (buffer, &start, opening_tag, -1);
     gtk_text_buffer_get_iter_at_mark (buffer, &end, mark);
     gtk_text_buffer_insert (buffer, &end, closing_tag, -1);
-  }
-  else {
 
-    gtk_text_buffer_get_iter_at_mark (buffer, &end, gtk_text_buffer_get_insert (buffer));
-    gtk_text_buffer_insert (buffer, &end, tags, -1);
+    /* restore the original selection */
+    gtk_text_buffer_get_iter_at_mark (buffer, &iter_sav_bound, mark_at_bound);
+    gtk_text_buffer_get_iter_at_mark (buffer, &iter_sav_insert, mark_at_insert);
+    gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &iter_sav_bound);
+    gtk_text_buffer_move_mark_by_name (buffer, "insert", &iter_sav_insert);
   }
+  else { /* no text selected - just insert */
+
+    gtk_text_buffer_get_iter_at_mark (buffer, &end,
+				      gtk_text_buffer_get_insert (buffer));
+    gtk_text_buffer_insert (buffer, &end, tags, -1);
+    gtk_text_iter_backward_chars (&end, strlen (closing_tag));
+    gtk_text_buffer_place_cursor (buffer, &end);
+  }
+
+  g_free (tags);
 
   gtk_widget_grab_focus (self->priv->message);
 }
@@ -536,6 +556,11 @@ message_activated_cb (G_GNUC_UNUSED GtkWidget *w,
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->priv->message));
     gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (buffer), &start_iter);
     gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (buffer), &end_iter);
+
+    /* if nothing to send - send nothing ;-) */
+    if (gtk_text_iter_get_offset (&end_iter) == 0)
+      return TRUE;
+
     body = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (buffer), &start_iter, &end_iter, TRUE);
 
     if (self->priv->chat->send_message (body))
@@ -553,6 +578,31 @@ on_chat_removed (ChatArea* self)
   gtk_widget_hide (self->priv->message);
 }
 
+static void
+on_chat_area_grab_focus (GtkWidget* widget,
+			 G_GNUC_UNUSED gpointer data)
+{
+  ChatArea* self = NULL;
+
+  self = (ChatArea*)widget;
+
+  gtk_widget_grab_focus (self->priv->message);
+}
+
+static gboolean
+on_chat_area_focus (GtkWidget* widget,
+		    G_GNUC_UNUSED GtkDirectionType direction,
+		    G_GNUC_UNUSED gpointer data)
+{
+  ChatArea* self = NULL;
+
+  self = (ChatArea*)widget;
+
+  gtk_widget_grab_focus (self->priv->message);
+
+  return TRUE;
+}
+
 /* GObject code */
 
 static void
@@ -829,34 +879,8 @@ chat_area_init (GTypeInstance* instance,
   GtkWidget* vbox = NULL;
   GtkWidget* bbox = NULL;
   GtkWidget* button = NULL;
-  const gchar** smileys = gm_get_smileys ();
-  gint smiley;
-  GdkPixbuf* pixbuf = NULL;
-  GtkWidget* image = NULL;
-  GtkWidget* smiley_item = NULL;
-  GtkWidget* smiley_button = NULL;
-
-  /* we need to build a nice menu for smileys */
-  self->priv->smiley_menu = gtk_menu_new ();
-  g_object_ref (self->priv->smiley_menu);
-  for (smiley = 0;
-       smileys[smiley] != NULL;
-       smiley = smiley + 2) {
-
-    smiley_item = gtk_image_menu_item_new_with_label (smileys[smiley]);
-    pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-				       smileys[smiley + 1], 16,
-				       (GtkIconLookupFlags)0, NULL);
-    image = gtk_image_new_from_pixbuf (pixbuf);
-    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (smiley_item),
-				   image);
-    gtk_widget_show_all (smiley_item);
-    gtk_menu_shell_append (GTK_MENU_SHELL (self->priv->smiley_menu),
-			   smiley_item);
-
-    g_signal_connect (G_OBJECT (smiley_item), "activate",
-		      G_CALLBACK (on_smiley_activated), self);
-  }
+  GtkWidget* smiley_button;
+  GtkWidget* smiley_chooser_button = NULL;
 
   frame = gtk_frame_new (NULL);
   vbox = gtk_vbox_new (FALSE, 2);
@@ -875,18 +899,24 @@ chat_area_init (GTypeInstance* instance,
 		      FALSE, FALSE, 2);
   gtk_widget_show (bbox);
 
-  button = gtk_button_new_with_mnemonic (_("_Smile..."));
-  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   smiley_button = gtk_image_new_from_icon_name ("face-smile", GTK_ICON_SIZE_BUTTON);
-  gtk_button_set_image (GTK_BUTTON(button), smiley_button);
-  gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
-  g_signal_connect (G_OBJECT (button), "clicked",
-		    G_CALLBACK (on_smiley_clicked), self);
-  gtk_box_pack_start (GTK_BOX (bbox), button,
-		      FALSE, TRUE, 2);
-  gtk_widget_show (button);
 
+  smiley_chooser_button = gm_smiley_chooser_button_new ();
+  gtk_button_set_label (GTK_BUTTON(smiley_chooser_button), _("_Smile..."));
+  gtk_button_set_image (GTK_BUTTON(smiley_chooser_button), smiley_button);
+  gtk_button_set_relief (GTK_BUTTON(smiley_chooser_button), GTK_RELIEF_NONE);
+  gtk_button_set_focus_on_click (GTK_BUTTON (smiley_chooser_button), FALSE);
+  g_signal_connect (G_OBJECT (smiley_chooser_button), "smiley_selected",
+		    G_CALLBACK (on_smiley_selected), self);
+  gtk_box_pack_start (GTK_BOX (bbox), smiley_chooser_button,
+		      FALSE, TRUE, 2);
+
+  /* the BOLD button */
   button = gtk_button_new_from_stock (GTK_STOCK_BOLD);
+  g_object_set_data_full (G_OBJECT (button), "gm_open_tag",
+			  (gpointer) "<b>", NULL);
+  g_object_set_data_full (G_OBJECT (button), "gm_close_tag",
+			  (gpointer) "</b>", NULL);
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
   g_signal_connect (G_OBJECT (button), "clicked",
@@ -895,7 +925,12 @@ chat_area_init (GTypeInstance* instance,
 		      FALSE, TRUE, 2);
   gtk_widget_show (button);
 
+  /* the ITALIC button */
   button = gtk_button_new_from_stock (GTK_STOCK_ITALIC);
+  g_object_set_data_full (G_OBJECT (button), "gm_open_tag",
+			  (gpointer) "<i>", NULL);
+  g_object_set_data_full (G_OBJECT (button), "gm_close_tag",
+			  (gpointer) "</i>", NULL);
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
   g_signal_connect (G_OBJECT (button), "clicked",
@@ -904,7 +939,12 @@ chat_area_init (GTypeInstance* instance,
 		      FALSE, TRUE, 2);
   gtk_widget_show (button);
 
+  /* the UNDERLINE button */
   button = gtk_button_new_from_stock (GTK_STOCK_UNDERLINE);
+  g_object_set_data_full (G_OBJECT (button), "gm_open_tag",
+			  (gpointer) "<u>", NULL);
+  g_object_set_data_full (G_OBJECT (button), "gm_close_tag",
+			  (gpointer) "</u>", NULL);
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
   g_signal_connect (G_OBJECT (button), "clicked",
@@ -927,6 +967,12 @@ chat_area_init (GTypeInstance* instance,
 
   gtk_widget_set_size_request (GTK_WIDGET (vbox), 175, -1);
   gtk_widget_show_all (vbox);
+
+  g_signal_connect (G_OBJECT (self), "focus",
+		    G_CALLBACK (on_chat_area_focus), NULL);
+  g_signal_connect (G_OBJECT (self), "grab-focus",
+		    G_CALLBACK (on_chat_area_grab_focus), self->priv->message);
+  gtk_widget_grab_focus (self->priv->message);
 }
 
 
