@@ -27,7 +27,7 @@
 
 
 /*
- *                         gmconf-glib.c  -  description 
+ *                         gmconf-glib.c  -  description
  *                         ------------------------------------------
  *   begin                : Mar 2004
  *   copyright            : (C) 2004 by Julien Puydt
@@ -37,13 +37,15 @@
 
 #include <glib.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/stat.h>
 
 #include "toolbox/toolbox.h"
-#include "gmconf.h" 
+#include "gmconf.h"
 
 #ifdef WIN32
-#include "../winpaths.h"
+#include "platform/winpaths.h"
 #endif
 
 
@@ -62,6 +64,30 @@
  * - the implementation of gmconf.h's api.
  */
 
+#define check_entry_type_return(entry,_type,val) G_STMT_START{ \
+  if (G_LIKELY(entry != NULL && entry->type == _type)) \
+    {} \
+  else { \
+    gchar *msg = NULL; \
+    msg = g_strdup_printf ("gmconf key %s is of type " #_type, entry->key); \
+    g_return_if_fail_warning (G_LOG_DOMAIN, __PRETTY_FUNCTION__, msg); \
+    g_free (msg); \
+    return val; \
+  } \
+}G_STMT_END
+
+#define check_entry_for_key_return(entry,key,val) G_STMT_START{	\
+  if (G_LIKELY(entry != NULL)) \
+    {} \
+  else { \
+    gchar *msg = NULL; \
+    msg = g_strdup_printf ("key %s has no corresponding entry! ", key); \
+    g_return_if_fail_warning (G_LOG_DOMAIN, __PRETTY_FUNCTION__, msg); \
+    g_free (msg); \
+    return val; \
+  } \
+}G_STMT_END
+
 
 /* the data types used in this file */
 
@@ -81,13 +107,13 @@ typedef struct _DataBase
 typedef struct _Notifier
 {
   GmConfNotifier func;
-  gpointer data;  
+  gpointer data;
 } Notifier;
 
 /* that implementation uses the GM_CONF_OTHER type of entry (the one that is
  * free to use for the specific implementation) to store "redirections", which
  * is the scheme that is used to make notifications propagate to parents in the
- * namespace 
+ * namespace
  */
 struct _GmConfEntry
 {
@@ -134,7 +160,7 @@ typedef struct _SchParser
 /* this little structure is needed to get enough data through the
  * iterator when destroying a full namespace
  */
-typedef struct _NamespcWrapper 
+typedef struct _NamespcWrapper
 {
   GData **datalist;
   const gchar *namespc;
@@ -158,7 +184,7 @@ static GSList *list_from_string (const gchar *);
 
 /* notifier functions */
 static Notifier *notifier_new (const GmConfNotifier, const gpointer);
-static void notifier_destroy (Notifier *); 
+static void notifier_destroy (Notifier *);
 static void notifier_destroy_in_list (gpointer elt, gpointer unused);
 static void notifier_call_on_entry (Notifier *, GmConfEntry *);
 
@@ -169,7 +195,7 @@ static void entry_destroy (gpointer);
 static const gchar *entry_get_key (const GmConfEntry *);
 static void entry_set_key (GmConfEntry *, const gchar *);
 
-static const GmConfEntryType entry_get_type (const GmConfEntry *);
+static GmConfEntryType entry_get_type (const GmConfEntry *);
 static void entry_set_type (GmConfEntry *, const GmConfEntryType);
 
 static gboolean entry_get_bool (const GmConfEntry *);
@@ -210,7 +236,7 @@ static void  sch_parser_end_element (GMarkupParseContext *context,
 				     GError **error);
 static void sch_parser_characters (GMarkupParseContext *context,
 				   const gchar *text,
-				   gsize text_len,  
+				   gsize text_len,
 				   gpointer data,
 				   GError **error);
 
@@ -232,7 +258,7 @@ static void database_remove_namespace_in_datalist (GQuark key_id,
 						   gpointer user_data);
 static void database_remove_namespace (DataBase *, const gchar *);
 static GmConfEntry *database_get_entry_for_key (DataBase *, const gchar *);
-static GmConfEntry *database_get_entry_for_key_create (DataBase *, 
+static GmConfEntry *database_get_entry_for_key_create (DataBase *,
 						       const gchar *);
 
 static void database_set_watched (DataBase *, const gboolean);
@@ -258,7 +284,7 @@ string_list_deep_copy (const GSList *orig)
   return result;
 }
 
-static void 
+static void
 string_list_deep_destroy (GSList *list)
 {
   GSList *ptr = NULL;
@@ -438,16 +464,17 @@ list_from_string (const gchar *str)
     item = NULL;
   }
 
-    
+
   return result;
 }
 
 /* implementation of notifier functions */
 static Notifier *
-notifier_new (const GmConfNotifier func, const gpointer data)
+notifier_new (const GmConfNotifier func,
+	      const gpointer data)
 {
   Notifier *notifier = NULL;
-  
+
   g_return_val_if_fail (func != NULL, NULL);
 
   notifier = g_new (Notifier, 1);
@@ -463,13 +490,15 @@ notifier_destroy (Notifier *notifier)
 }
 
 static void
-notifier_destroy_in_list (gpointer elt, gpointer unused)
+notifier_destroy_in_list (gpointer elt,
+			  G_GNUC_UNUSED gpointer unused)
 {
   notifier_destroy ((Notifier *)elt);
 }
 
 static void
-notifier_call_on_entry (Notifier *notifier, GmConfEntry *entry)
+notifier_call_on_entry (Notifier *notifier,
+			GmConfEntry *entry)
 {
   g_return_if_fail (notifier != NULL);
   g_return_if_fail (entry != NULL);
@@ -519,6 +548,8 @@ entry_destroy (gpointer ent)
   case GM_CONF_LIST:
     string_list_deep_destroy (entry->value.list);
     break;
+  default:
+    break;
   }
 
   g_slist_foreach (entry->notifiers, notifier_destroy_in_list, NULL);
@@ -530,20 +561,23 @@ static const gchar *
 entry_get_key (const GmConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, NULL);
-  
+
   return entry->key;
 }
 
 static void
-entry_set_key (GmConfEntry *entry, const gchar *key)
+entry_set_key (GmConfEntry *entry,
+	       const gchar *key)
 {
   g_return_if_fail (entry != NULL);
   g_return_if_fail (key != NULL);
 
+  if (entry->key)
+    g_free (entry->key);
   entry->key = g_strdup (key);
 }
 
-static const GmConfEntryType
+static GmConfEntryType
 entry_get_type (const GmConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, GM_CONF_OTHER);
@@ -552,7 +586,8 @@ entry_get_type (const GmConfEntry *entry)
 }
 
 static void
-entry_set_type (GmConfEntry *entry, const GmConfEntryType type)
+entry_set_type (GmConfEntry *entry,
+		const GmConfEntryType type)
 {
   g_return_if_fail (entry != NULL);
 
@@ -562,14 +597,14 @@ entry_set_type (GmConfEntry *entry, const GmConfEntryType type)
 static gboolean
 entry_get_bool (const GmConfEntry *entry)
 {
-  g_return_val_if_fail (entry != NULL, FALSE);
-  g_return_val_if_fail (entry->type == GM_CONF_BOOL, FALSE);
+  check_entry_type_return (entry, GM_CONF_BOOL, FALSE);
 
   return entry->value.boolean;
 }
 
 static void
-entry_set_bool (GmConfEntry *entry, const gboolean val)
+entry_set_bool (GmConfEntry *entry,
+		const gboolean val)
 {
   g_return_if_fail (entry != NULL);
 
@@ -580,13 +615,14 @@ entry_set_bool (GmConfEntry *entry, const gboolean val)
 static gint
 entry_get_int (const GmConfEntry *entry)
 {
-  g_return_val_if_fail (entry != NULL, 0);
-  g_return_val_if_fail (entry->type == GM_CONF_INT, 0);
+  check_entry_type_return (entry, GM_CONF_INT, 0);
 
   return entry->value.integer;
 }
 
-static void entry_set_int (GmConfEntry *entry, const gint val)
+static void
+entry_set_int (GmConfEntry *entry,
+	       const gint val)
 {
   g_return_if_fail (entry != NULL);
 
@@ -597,14 +633,14 @@ static void entry_set_int (GmConfEntry *entry, const gint val)
 static gfloat
 entry_get_float (const GmConfEntry *entry)
 {
-  g_return_val_if_fail (entry != NULL, 0);
-  g_return_val_if_fail (entry->type == GM_CONF_FLOAT, 0);
+  check_entry_type_return (entry, GM_CONF_FLOAT, 0);
 
   return entry->value.floa;
 }
 
 static void
-entry_set_float (GmConfEntry *entry, const gfloat val)
+entry_set_float (GmConfEntry *entry,
+		 const gfloat val)
 {
   g_return_if_fail (entry != NULL);
 
@@ -615,8 +651,7 @@ entry_set_float (GmConfEntry *entry, const gfloat val)
 static const gchar *
 entry_get_string (const GmConfEntry *entry)
 {
-  g_return_val_if_fail (entry != NULL, 0);
-  g_return_val_if_fail (entry->type == GM_CONF_STRING, 0);
+  check_entry_type_return (entry, GM_CONF_STRING, NULL);
 
   return entry->value.string;
 }
@@ -633,14 +668,14 @@ entry_set_string (GmConfEntry *entry, const gchar *val)
 static GSList *
 entry_get_list (const GmConfEntry *entry)
 {
-  g_return_val_if_fail (entry != NULL, 0);
-  g_return_val_if_fail (entry->type == GM_CONF_LIST, 0);
+  check_entry_type_return (entry, GM_CONF_LIST, NULL);
 
   return entry->value.list;
 }
 
 static void
-entry_set_list (GmConfEntry *entry, GSList *val)
+entry_set_list (GmConfEntry *entry,
+		GSList *val)
 {
   g_return_if_fail (entry != NULL);
 
@@ -648,8 +683,9 @@ entry_set_list (GmConfEntry *entry, GSList *val)
   entry->value.list = string_list_deep_copy (val);
 }
 
-static void 
-entry_set_redirect (GmConfEntry *entry, GmConfEntry *redirect)
+static void
+entry_set_redirect (GmConfEntry *entry,
+		    GmConfEntry *redirect)
 {
   g_return_if_fail (entry != NULL);
 
@@ -669,7 +705,7 @@ entry_call_notifiers_from_g_idle (gpointer data)
   entry = (GmConfEntry *)data;
   for (ptr = entry->notifiers; ptr != NULL; ptr = ptr->next) {
     notif = (Notifier *)ptr->data;
-    if (entry->type == GM_CONF_OTHER)
+    if (entry->type == GM_CONF_OTHER && entry->value.redirect != NULL)
       notifier_call_on_entry (notif, entry->value.redirect);
     else
       notifier_call_on_entry (notif, entry);
@@ -677,7 +713,7 @@ entry_call_notifiers_from_g_idle (gpointer data)
   return FALSE;
 }
 
-static void 
+static void
 entry_call_notifiers (const GmConfEntry *entry)
 {
   g_return_if_fail (entry != NULL);
@@ -687,7 +723,9 @@ entry_call_notifiers (const GmConfEntry *entry)
 }
 
 static gpointer
-entry_add_notifier (GmConfEntry *entry, GmConfNotifier func, gpointer data)
+entry_add_notifier (GmConfEntry *entry,
+		    GmConfNotifier func,
+		    gpointer data)
 {
   Notifier *notif = NULL;
 
@@ -701,16 +739,17 @@ entry_add_notifier (GmConfEntry *entry, GmConfNotifier func, gpointer data)
 
 
 static void
-entry_remove_notifier (GmConfEntry *entry, gpointer identifier)
+entry_remove_notifier (GmConfEntry *entry,
+		       gpointer identifier)
 {
-  /* no check, since the only function calling here is 
+  /* no check, since the only function calling here is
      entry_remove_notifier_in_list */
 
   entry->notifiers = g_slist_remove (entry->notifiers, identifier);
 }
 
-static void 
-entry_remove_notifier_in_list (GQuark unused,
+static void
+entry_remove_notifier_in_list (G_GNUC_UNUSED GQuark unused,
 			       gpointer entry,
 			       gpointer identifier)
 {
@@ -733,6 +772,13 @@ database_new ()
   return db;
 }
 
+static void
+database_destroy (DataBase *db)
+{
+  g_datalist_clear (&db->entries);
+  g_free (db);
+}
+
 static DataBase *
 database_get_default ()
 {
@@ -742,13 +788,13 @@ database_get_default ()
   return db;
 }
 
-static void 
-sch_parser_start_element (GMarkupParseContext *context,
+static void
+sch_parser_start_element (G_GNUC_UNUSED GMarkupParseContext *context,
 			  const gchar *element_name,
-			  const gchar **attribute_names,
-			  const gchar **attribute_values,
+			  G_GNUC_UNUSED const gchar **attribute_names,
+			  G_GNUC_UNUSED const gchar **attribute_values,
 			  gpointer data,
-			  GError **error)
+			  G_GNUC_UNUSED GError **error)
 {
   SchParser *parser = NULL;
 
@@ -766,11 +812,11 @@ sch_parser_start_element (GMarkupParseContext *context,
     parser->state = VALUE;
 }
 
-static void  
-sch_parser_end_element (GMarkupParseContext *context,
+static void
+sch_parser_end_element (G_GNUC_UNUSED GMarkupParseContext *context,
 			const gchar *element_name,
 			gpointer data,
-			GError **error)
+			G_GNUC_UNUSED GError **error)
 {
   SchParser *parser = NULL;
 
@@ -787,12 +833,12 @@ sch_parser_end_element (GMarkupParseContext *context,
   parser->state = START;
 }
 
-static void 
-sch_parser_characters (GMarkupParseContext *context,
+static void
+sch_parser_characters (G_GNUC_UNUSED GMarkupParseContext *context,
 		       const gchar *text,
-		       gsize text_len,  
+		       G_GNUC_UNUSED gsize text_len,
 		       gpointer data,
-		       GError **error)
+		       G_GNUC_UNUSED GError **error)
 {
   SchParser *parser = NULL;
 
@@ -801,6 +847,7 @@ sch_parser_characters (GMarkupParseContext *context,
   parser = (SchParser *)data;
 
   switch (parser->state) {
+
   case  START:
     /* we're not interested in that data: just eat it! */
     break;
@@ -817,7 +864,7 @@ sch_parser_characters (GMarkupParseContext *context,
     else if (strcmp (text, "string") == 0)
       entry_set_type (parser->entry, GM_CONF_STRING);
     else if (strcmp (text, "list") == 0)
-      entry_set_type (parser->entry, GM_CONF_LIST);   
+      entry_set_type (parser->entry, GM_CONF_LIST);
     else
       entry_set_type (parser->entry, GM_CONF_OTHER);
     break;
@@ -838,14 +885,20 @@ sch_parser_characters (GMarkupParseContext *context,
     case GM_CONF_LIST:
       entry_set_list (parser->entry, list_from_string (text));
       break;
+    case GM_CONF_OTHER: /* shouldn't happen */
+      break;
     default:
       break;
     }
+
+  default:
+    ;/* nothing */
   }
 }
 
 static gboolean
-database_load_file (DataBase *db, const gchar *filename)
+database_load_file (DataBase *db,
+		    const gchar *filename)
 {
   SchParser *parser = NULL;
   GMarkupParseContext *context = NULL;
@@ -858,7 +911,7 @@ database_load_file (DataBase *db, const gchar *filename)
   g_return_val_if_fail (filename != NULL, FALSE);
 
   io = g_io_channel_new_file (filename, "r", NULL);
-  if (!io) 
+  if (!io)
     return FALSE;
   parser = g_new (SchParser, 1);
   parser->state = START;
@@ -881,6 +934,7 @@ database_load_file (DataBase *db, const gchar *filename)
           return g_markup_parse_context_end_parse (context, NULL);
         case G_IO_STATUS_NORMAL:
         case G_IO_STATUS_AGAIN:
+	default:
           break;
         }
       if (!g_markup_parse_context_parse (context, buffer, len, NULL))
@@ -894,14 +948,15 @@ database_load_file (DataBase *db, const gchar *filename)
   return TRUE;
 }
 
-static void 
-database_save_entry (GQuark quark, gpointer data,
+static void
+database_save_entry (G_GNUC_UNUSED GQuark quark,
+		     gpointer data,
 		     gpointer user_data)
 {
   GmConfEntry *entry = NULL;
-  GSList *ptr;
   GIOChannel *io = NULL;
   gchar *value = NULL;
+  const gchar *txt = NULL;
 
   g_return_if_fail (data != NULL);
   g_return_if_fail (user_data != NULL);
@@ -955,7 +1010,11 @@ database_save_entry (GQuark quark, gpointer data,
     value = string_from_float (entry_get_float (entry));
     break;
   case GM_CONF_STRING:
-    value = g_strdup (entry_get_string (entry));
+    txt = entry_get_string (entry);
+    if (txt != NULL)
+      value = g_markup_escape_text (txt, -1);
+    else
+      value = g_strdup ("");
     break;
   case GM_CONF_LIST:
     value = string_from_list (entry_get_list (entry));
@@ -972,18 +1031,19 @@ database_save_entry (GQuark quark, gpointer data,
   g_io_channel_write_chars (io, "</schema>\n", -1, NULL, NULL);
 }
 
-static gboolean 
-database_save_file (DataBase *db, const gchar *filename)
+static gboolean
+database_save_file (DataBase *db,
+		    const gchar *filename)
 {
   GIOChannel *io = NULL;
   gchar *dirname = NULL;
 
-  g_return_if_fail (db != NULL);
-  g_return_if_fail (filename != NULL);
+  g_return_val_if_fail (db != NULL, FALSE);
+  g_return_val_if_fail (filename != NULL, FALSE);
 
   dirname = g_path_get_dirname (filename);
   if (!g_file_test (dirname, G_FILE_TEST_IS_DIR)) {
-    if (!gm_mkdir_with_parents (dirname, S_IRWXU))
+    if (!g_mkdir_with_parents (dirname, S_IRWXU))
       g_warning ("Unable to create directory %s\n", dirname);
   }
   g_free (dirname);
@@ -1001,8 +1061,9 @@ database_save_file (DataBase *db, const gchar *filename)
   return TRUE;
 }
 
-static void 
-database_add_entry (DataBase *db, GmConfEntry *entry)
+static void
+database_add_entry (DataBase *db,
+		    GmConfEntry *entry)
 {
   g_return_if_fail (db != NULL);
   g_return_if_fail (entry != NULL);
@@ -1011,15 +1072,15 @@ database_add_entry (DataBase *db, GmConfEntry *entry)
 			    entry, entry_destroy);
 }
 
-static void 
-database_remove_namespace_in_datalist (GQuark key_id,
+static void
+database_remove_namespace_in_datalist (G_GNUC_UNUSED GQuark key_id,
 				       gpointer data,
 				       gpointer user_data)
 {
   GmConfEntry *entry = NULL;
   NamespcWrapper *wrapper = NULL;
   const gchar *key = NULL;
-  
+
   g_return_if_fail (data != NULL);
   g_return_if_fail (user_data != NULL);
 
@@ -1032,24 +1093,26 @@ database_remove_namespace_in_datalist (GQuark key_id,
 
 }
 
-static void 
-database_remove_namespace (DataBase *db, const gchar *namespc)
+static void
+database_remove_namespace (DataBase *db,
+			   const gchar *namespc)
 {
   NamespcWrapper *wrapper = NULL;
-  
+
   g_return_if_fail (db != NULL);
   g_return_if_fail (namespc != NULL);
 
   wrapper = g_new (NamespcWrapper, 1);
   wrapper->datalist = &db->entries;
   wrapper->namespc = namespc;
-  g_datalist_foreach (&db->entries, 
+  g_datalist_foreach (&db->entries,
 		      database_remove_namespace_in_datalist, wrapper);
   g_free (wrapper);
 }
 
 static GmConfEntry *
-database_get_entry_for_key (DataBase *db, const gchar *key)
+database_get_entry_for_key (DataBase *db,
+			    const gchar *key)
 {
   GmConfEntry *entry = NULL;
 
@@ -1057,11 +1120,13 @@ database_get_entry_for_key (DataBase *db, const gchar *key)
   g_return_val_if_fail (key != NULL, NULL);
 
   entry = g_datalist_get_data (&db->entries, key);
+
   return entry;
 }
 
 static GmConfEntry *
-database_get_entry_for_key_create (DataBase *db, const gchar *key)
+database_get_entry_for_key_create (DataBase *db,
+				   const gchar *key)
 {
   GmConfEntry *entry = NULL;
 
@@ -1077,40 +1142,42 @@ database_get_entry_for_key_create (DataBase *db, const gchar *key)
   return entry;
 }
 
-static void 
-database_set_watched (DataBase *db, const gboolean bool)
+static void
+database_set_watched (DataBase *db,
+		      const gboolean bool)
 {
   g_return_if_fail (db != NULL);
 
   db->is_watched = bool;
 }
 
-static void 
-database_notify_on_namespace (DataBase *db, const gchar *namespac)
+static void
+database_notify_on_namespace (DataBase *db,
+			      const gchar *namespac)
 {
   GmConfEntry *parent_entry = NULL, *entry = NULL;
   gchar *key = NULL;
-  
+
 
   g_return_if_fail (db != NULL);
   g_return_if_fail (namespac != NULL);
   g_return_if_fail (namespac[0] == '/'); /* that makes the loop work! */
 
   entry = database_get_entry_for_key (db, namespac);
-  
-  g_return_if_fail (entry != NULL);  
-  
+
+  g_return_if_fail (entry != NULL);
+
   if (db->is_watched == FALSE)
     return;
 
   for (key = g_strdup (namespac);
-       key[0] != 0; 
+       key[0] != 0;
        g_strrstr (key, "/")[0] = 0) {
     parent_entry = database_get_entry_for_key (db, key);
     if (parent_entry != NULL) {
       if (entry_get_type (parent_entry) == GM_CONF_OTHER)
 	entry_set_redirect (parent_entry, entry);
-      entry_call_notifiers (parent_entry);	
+      entry_call_notifiers (parent_entry);
       if (entry_get_type (entry) == GM_CONF_OTHER)
 	entry_set_redirect (parent_entry, NULL);
     }
@@ -1139,6 +1206,12 @@ gm_conf_load_user_conf (DataBase *db)
   filename = gm_conf_get_user_conf_filename ();
   result = database_load_file (db, filename);
 
+  if (G_LIKELY (result))
+    {}
+  else
+    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+	   "couldn't read the user configuration in %s", filename);
+
   g_free (filename);
 
   return result;
@@ -1156,6 +1229,12 @@ gm_conf_load_sys_conf (DataBase *db)
   filename = g_build_filename (SYSCONFDIR, "ekiga",
 			       "ekiga.schemas", NULL);
   result = database_load_file (db, filename);
+
+  if (G_LIKELY (result))
+    {}
+  else
+    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+	   "couldn't read the system configuration in %s", filename);
   g_free (filename);
 
   return result;
@@ -1164,7 +1243,7 @@ gm_conf_load_sys_conf (DataBase *db)
 
 /* last but not least, the implementation of the gmconf.h api */
 static gboolean
-saveconf_timer_callback (gpointer unused)
+saveconf_timer_callback (G_GNUC_UNUSED gpointer unused)
 {
   DataBase *db = database_get_default ();
   gchar *user_conf = NULL;
@@ -1173,16 +1252,16 @@ saveconf_timer_callback (gpointer unused)
   database_save_file (db, user_conf);
 
   g_free (user_conf);
-  
+
   return TRUE;
 }
 
-void 
-gm_conf_init (int argc, char **argv)
+void
+gm_conf_init ()
 {
   gboolean result = FALSE;
   DataBase *db = database_get_default ();
- 
+
   result = gm_conf_load_sys_conf (db);
   result = (gm_conf_load_user_conf (db) || result);
   if (!result)
@@ -1196,7 +1275,17 @@ gm_conf_init (int argc, char **argv)
 
 }
 
-void 
+
+void
+gm_conf_shutdown ()
+{
+  /* a crash is sure to happen if anyone uses gmconf after this... */
+  DataBase *db = database_get_default ();
+  database_destroy (db);
+}
+
+
+void
 gm_conf_save ()
 {
   DataBase *db = database_get_default ();
@@ -1209,7 +1298,7 @@ gm_conf_save ()
   g_free (user_conf);
 }
 
-void 
+void
 gm_conf_watch ()
 {
   DataBase *db = database_get_default ();
@@ -1217,7 +1306,7 @@ gm_conf_watch ()
   database_set_watched (db, TRUE);
 }
 
-void 
+void
 gm_conf_unwatch ()
 {
   DataBase *db = database_get_default ();
@@ -1225,23 +1314,23 @@ gm_conf_unwatch ()
   database_set_watched (db, FALSE);
 }
 
-gpointer 
-gm_conf_notifier_add (const gchar *namespac, 
+gpointer
+gm_conf_notifier_add (const gchar *namespac,
 		      GmConfNotifier func,
 		      gpointer user_data)
 {
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
-  
+
   g_return_val_if_fail (namespac != NULL, NULL);
   g_return_val_if_fail (func != NULL, NULL);
-  
+
   entry = database_get_entry_for_key_create (db, namespac);
 
   return entry_add_notifier (entry, func, user_data);
 }
 
-void 
+void
 gm_conf_notifier_remove (gpointer identifier)
 {
   DataBase *db = database_get_default ();
@@ -1263,7 +1352,7 @@ gm_conf_notifier_trigger (const gchar *namespac)
   database_notify_on_namespace (db, namespac);
 }
 
-GmConfEntryType 
+GmConfEntryType
 gm_conf_entry_get_type (GmConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, GM_CONF_OTHER);
@@ -1279,7 +1368,7 @@ gm_conf_entry_get_key (GmConfEntry *entry)
   return entry_get_key (entry);
 }
 
-gboolean 
+gboolean
 gm_conf_entry_get_bool (GmConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, FALSE);
@@ -1287,7 +1376,7 @@ gm_conf_entry_get_bool (GmConfEntry *entry)
   return entry_get_bool (entry);
 }
 
-gint 
+gint
 gm_conf_entry_get_int (GmConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, 0);
@@ -1303,16 +1392,17 @@ gm_conf_entry_get_string (GmConfEntry *entry)
   return entry_get_string (entry);
 }
 
-const GSList *
+GSList *
 gm_conf_entry_get_list (GmConfEntry *entry)
 {
   g_return_val_if_fail (entry != NULL, NULL);
 
-  return entry_get_list (entry);
+  return string_list_deep_copy (entry_get_list (entry));
 }
 
-void 
-gm_conf_set_bool (const gchar *key, const gboolean val)
+void
+gm_conf_set_bool (const gchar *key,
+		  const gboolean val)
 {
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
@@ -1327,22 +1417,24 @@ gm_conf_set_bool (const gchar *key, const gboolean val)
   database_notify_on_namespace (db, entry_get_key (entry));
 }
 
-gboolean 
+gboolean
 gm_conf_get_bool (const gchar *key)
 {
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
 
-  g_return_if_fail (key != NULL);
+  g_return_val_if_fail (key != NULL, FALSE);
 
   entry = database_get_entry_for_key (db, key);
-  if (entry == NULL)
-    return FALSE;
+
+  check_entry_for_key_return (entry, key, FALSE);
+
   return entry_get_bool (entry);
 }
 
-void 
-gm_conf_set_int (const gchar *key, const int val)
+void
+gm_conf_set_int (const gchar *key,
+		 const int val)
 {
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
@@ -1357,22 +1449,24 @@ gm_conf_set_int (const gchar *key, const int val)
   database_notify_on_namespace (db, entry_get_key (entry));
 }
 
-int 
+int
 gm_conf_get_int (const gchar *key)
 {
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
 
-  g_return_if_fail (key != NULL);
+  g_return_val_if_fail (key != NULL, 0);
 
   entry = database_get_entry_for_key (db, key);
-  if (entry == NULL)
-    return 0;
+
+  check_entry_for_key_return (entry, key, 0);
+
   return entry_get_int (entry);
 }
 
-void 
-gm_conf_set_float (const gchar *key, const float val)
+void
+gm_conf_set_float (const gchar *key,
+		   const float val)
 {
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
@@ -1387,23 +1481,24 @@ gm_conf_set_float (const gchar *key, const float val)
   database_notify_on_namespace (db, entry_get_key (entry));
 }
 
-gfloat 
+gfloat
 gm_conf_get_float (const gchar *key)
 {
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
 
-  g_return_if_fail (key != NULL);
+  g_return_val_if_fail (key != NULL, 0);
 
   entry = database_get_entry_for_key (db, key);
-  if (entry == NULL)
-    return 0;
+
+  check_entry_for_key_return (entry, key, 0);
 
   return entry_get_float (entry);
 }
 
-void 
-gm_conf_set_string (const gchar *key, const gchar *val)
+void
+gm_conf_set_string (const gchar *key,
+		    const gchar *val)
 {
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
@@ -1424,16 +1519,18 @@ gm_conf_get_string (const gchar *key)
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
 
-  g_return_if_fail (key != NULL);
+  g_return_val_if_fail (key != NULL, NULL);
 
   entry = database_get_entry_for_key (db, key);
-  if (entry == NULL)
-    return NULL;
+
+  check_entry_for_key_return (entry, key, NULL);
+
   return g_strdup (entry_get_string (entry));
 }
 
-void 
-gm_conf_set_string_list (const gchar *key, GSList *val)
+void
+gm_conf_set_string_list (const gchar *key,
+			 GSList *val)
 {
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
@@ -1454,15 +1551,16 @@ gm_conf_get_string_list (const gchar *key)
   DataBase *db = database_get_default ();
   GmConfEntry *entry = NULL;
 
-  g_return_if_fail (key != NULL);
+  g_return_val_if_fail (key != NULL, NULL);
 
   entry = database_get_entry_for_key (db, key);
-  if (entry == NULL)
-    return NULL;
+
+  check_entry_for_key_return (entry, key, NULL);
+
   return string_list_deep_copy (entry_get_list (entry));
 }
 
-void 
+void
 gm_conf_destroy (const gchar *namespac)
 {
   DataBase *db = database_get_default ();
@@ -1472,7 +1570,7 @@ gm_conf_destroy (const gchar *namespac)
   database_remove_namespace (db, namespac);
 }
 
-gboolean 
+gboolean
 gm_conf_is_key_writable (const gchar *key)
 {
   g_return_val_if_fail (key != NULL, FALSE);
@@ -1481,13 +1579,15 @@ gm_conf_is_key_writable (const gchar *key)
 }
 
 gchar *
-gm_conf_escape_key (const gchar *key, gint len)
+gm_conf_escape_key (const gchar *key,
+		    G_GNUC_UNUSED gint len)
 {
   return g_strescape (key, NULL); /* we don't honor len */
 }
 
 gchar *
-gm_conf_unescape_key (const gchar *key, gint len)
+gm_conf_unescape_key (const gchar *key,
+		      G_GNUC_UNUSED gint len)
 {
   return g_strcompress (key); /* we don't honor len */
 }
