@@ -30,7 +30,8 @@
  *                         dbus_component.h  -  description
  *                         -----------------------------
  *   begin                : Tue Nov 1  2005
- *   copyright            : (C) 2005 by Julien Puydt
+ *   copyright            : (c) 2005 by Julien Puydt
+ *                          (c) 2007 by Damien Sandras
  *   description          : This files contains the interface to the DBUS
  *                          interface of gnomemeeting.
  *
@@ -40,53 +41,35 @@
 #define __DBUS_COMPONENT_H
 
 #include <glib-object.h>
-
-#include "manager.h"
+#include "framework/services.h"
 
 G_BEGIN_DECLS
 
-enum {
-  INVALID_ACCOUNT,
-  UNREGISTERED,
-  REGISTERED
+#define EKIGA_TYPE_DBUS_COMPONENT               (ekiga_dbus_component_get_type ())
+#define EKIGA_DBUS_COMPONENT(obj)               (G_TYPE_CHECK_INSTANCE_CAST ((obj), EKIGA_TYPE_DBUS_COMPONENT, EkigaDBusComponent))
+#define EKIGA_DBUS_COMPONENT_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST ((klass), EKIGA_TYPE_DBUS_COMPONENT, EkigaDBusComponentClass))
+#define EKIGA_IS_DBUS_COMPONENT(obj)            (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EKIGA_TYPE_DBUS_COMPONENT))
+#define EKIGA_IS_DBUS_COMPONENT_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE ((klass), EKIGA_TYPE_DBUS_COMPONENT))
+
+typedef struct _EkigaDBusComponentPrivate       EkigaDBusComponentPrivate;
+typedef struct _EkigaDBusComponent              EkigaDBusComponent;
+
+struct _EkigaDBusComponent {
+  GObject                    parent;
+  EkigaDBusComponentPrivate *priv;
 };
 
-enum {
-  INVALID_CALL,
-  STANDBY,
-  CALLING,
-  CONNECTED,
-  CALLED
-};
+typedef GObjectClass EkigaDBusComponentClass;
 
-GObject *gnomemeeting_dbus_component_new ();
+GType                ekiga_dbus_component_get_type ();
+EkigaDBusComponent  *ekiga_dbus_component_new (Ekiga::ServiceCore *core);
 
-void gnomemeeting_dbus_component_set_call_state (GObject *obj,
-						 const gchar *token,
-						 GMManager::CallingState state);
+gboolean             ekiga_dbus_claim_ownership ();
 
-void gnomemeeting_dbus_component_set_call_info (GObject *obj,
-						const gchar *token,
-						const gchar *name,
-						const gchar *client,
-						const gchar *url,
-						const gchar *protocol_prefix);
-
-void gnomemeeting_dbus_component_set_call_on_hold (GObject *obj,
-						   const gchar *token,
-						   gboolean is_on_hold);
-
-gboolean gnomemeting_dbus_component_is_first_instance (GObject *obj);
-
-void gnomemeeting_dbus_component_call (GObject *obj,
-				       const gchar *uri);
-
-void gnomemeeting_dbus_component_account_registration (GObject *obj,
-						       const gchar *username,
-						       const gchar *domain,
-						       gboolean registered);
-							      
+void                 ekiga_dbus_client_show ();
+void                 ekiga_dbus_client_connect (const gchar *uri);
 
 G_END_DECLS
 
 #endif /* __DBUS_COMPONENT_H */
+/* ex:set ts=2 sw=2 et: */
