@@ -53,13 +53,13 @@
 #include "opal-account.h"
 
 static void
-presence_status_in_main (Ekiga::PresenceCore* core,
+presence_status_in_main (Ekiga::PresenceFetcher* fetcher,
                          std::string uri,
                          std::string presence,
                          std::string status)
 {
-  core->presence_received.emit (uri, presence);
-  core->status_received.emit (uri, status);
+  fetcher->presence_received.emit (uri, presence);
+  fetcher->status_received.emit (uri, status);
 }
 
 static void
@@ -268,22 +268,22 @@ bool Opal::Sip::EndPoint::menu_builder_add_actions (const std::string& fullname,
        it++) {
 
     std::stringstream call_action;
-    std::stringstream forward_action;
+    std::stringstream transfer_action;
     if (!(*ita).empty ()) {
       call_action << _("Call") << " [" << (*ita) << "]";
-      forward_action << _("Transfer") << " [" << (*ita) << "]";
+      transfer_action << _("Transfer") << " [" << (*ita) << "]";
     }
     else {
       call_action << _("Call");
-      forward_action << _("Transfer");
+      transfer_action << _("Transfer");
     }
 
     if (0 == GetConnectionCount ())
       builder.add_action ("call", call_action.str (),
                           sigc::bind (sigc::mem_fun (this, &Opal::Sip::EndPoint::on_dial), (*it)));
     else 
-      builder.add_action ("transfer", forward_action.str (),
-                          sigc::bind (sigc::mem_fun (this, &Opal::Sip::EndPoint::on_forward), (*it)));
+      builder.add_action ("transfer", transfer_action.str (),
+                          sigc::bind (sigc::mem_fun (this, &Opal::Sip::EndPoint::on_transfer), (*it)));
 
     ita++;
   }
@@ -325,7 +325,7 @@ void Opal::Sip::EndPoint::fetch (const std::string _uri)
     // We are registered yet
     if (std::find (domains.begin (), domains.end (), domain) != domains.end ()) {
 
-      Subscribe (SIPSubscribe::Presence, 1800, PString (_uri.c_str ()));
+      Subscribe (SIPSubscribe::Presence, 300, PString (_uri.c_str ()));
       subscribed_uris.push_back (_uri);
     }
     else {
@@ -390,7 +390,7 @@ void Opal::Sip::EndPoint::publish (const Ekiga::PersonalDetails & details)
     data += "</tuple>\r\n";
     data += "</presence>\r\n";
 
-    Publish (to, data, 500); // TODO: allow to change the 500 
+    Publish (to, data, 300); // TODO: allow to change the 500 
   }
 }
 
@@ -472,9 +472,9 @@ bool Opal::Sip::EndPoint::set_listen_port (unsigned port)
 {
   unsigned udp_min, udp_max;
 
-  interface.protocol = "udp";
-  interface.voip_protocol = "sip";
-  interface.interface = "*";
+  listen_iface.protocol = "udp";
+  listen_iface.voip_protocol = "sip";
+  listen_iface.id = "*";
 
   manager.get_udp_ports (udp_min, udp_max);
 
@@ -492,7 +492,7 @@ bool Opal::Sip::EndPoint::set_listen_port (unsigned port)
 
         if (StartListeners (PStringArray (str.str ()))) {
 
-          interface.port = port;
+          listen_iface.port = port;
           return true;
         }
 
@@ -500,7 +500,7 @@ bool Opal::Sip::EndPoint::set_listen_port (unsigned port)
       }
     }
     else
-      interface.port = port;
+      listen_iface.port = port;
   }
 
   return false;
@@ -509,7 +509,7 @@ bool Opal::Sip::EndPoint::set_listen_port (unsigned port)
 
 const Ekiga::CallProtocolManager::Interface & Opal::Sip::EndPoint::get_listen_interface () const
 {
-  return interface;
+  return listen_iface;
 }
 
 
@@ -553,7 +553,7 @@ unsigned Opal::Sip::EndPoint::get_nat_binding_delay ()
 
 bool Opal::Sip::EndPoint::subscribe (const Opal::Account & account)
 {
-  if (account.get_protocol_name () != "SIP")
+  if (account.get_protocol_name () != "SIP" || account.is_active ())
     return false;
 
   new subscriber (account, *this);
@@ -563,7 +563,7 @@ bool Opal::Sip::EndPoint::subscribe (const Opal::Account & account)
 
 bool Opal::Sip::EndPoint::unsubscribe (const Opal::Account & account)
 {
-  if (account.get_protocol_name () != "SIP")
+  if (account.get_protocol_name () != "SIP" || !account.is_active ())
     return false;
 
   new subscriber (account, *this);
@@ -590,14 +590,26 @@ void Opal::Sip::EndPoint::ShutDown ()
 void Opal::Sip::EndPoint::Register (const Opal::Account & account)
 {
   std::stringstream aor;
+  std::string host = account.get_host ();
+  std::string::size_type loc = host.find (":", 0);
+  if (loc != std::string::npos)
+    host = host.substr (0, loc);
 
-  aor << account.get_username () << "@" << account.get_host ();
-  if (!SIPEndPoint::Register (account.get_host (),
-                              account.get_username (),
-                              account.get_authentication_username (),
-                              account.get_password (),
-                              PString::Empty (), 
-                              (account.is_enabled () ? account.get_timeout () : 0)))
+  if (account.get_username ().find ("@") == std::string::npos) 
+    aor << account.get_username () << "@" << host;
+  else
+    aor << account.get_username ();
+
+  SIPRegister::Params params;
+  params.m_addressOfRecord = aor.str ();
+  params.m_registrarAddress = account.get_host ();
+  params.m_authID = account.get_authentication_username ();
+  params.m_password = account.get_password ();
+  params.m_expire = (account.is_enabled () ? account.get_timeout () : 0);
+  params.m_minRetryTime = 0;
+  params.m_maxRetryTime = 0;
+
+  if (!SIPEndPoint::Register (params))
     OnRegistrationFailed (aor.str (), SIP_PDU::MaxStatusCode, account.is_enabled ());
 }
 
@@ -649,7 +661,7 @@ void Opal::Sip::EndPoint::OnRegistered (const PString & _aor,
         found = (*iter).find (server, 0);
         if (found != string::npos) {
 
-          Subscribe (SIPSubscribe::Presence, 500, PString ((*iter).c_str ()));
+          Subscribe (SIPSubscribe::Presence, 300, PString ((*iter).c_str ()));
           subscribed_uris.push_back (*iter);
           to_subscribe_uris.remove (*iter++);
         }
@@ -664,7 +676,7 @@ void Opal::Sip::EndPoint::OnRegistered (const PString & _aor,
         found = (*iter).find (server, 0);
         if (found != string::npos) {
 
-          Subscribe (SIPSubscribe::Presence, 0, PString ((*iter).c_str ()));
+          Unsubscribe (SIPSubscribe::Presence, PString ((*iter).c_str ()));
           to_subscribe_uris.push_back (*iter);
           subscribed_uris.remove (*iter++);
         }
@@ -791,6 +803,7 @@ void Opal::Sip::EndPoint::OnRegistrationFailed (const PString & _aor,
     break;
 
   case SIP_PDU::Failure_BadExtension:
+    // Translators : The extension we are trying to register does not exist 
     info = _("Bad extension");
     break;
 
@@ -919,6 +932,20 @@ void Opal::Sip::EndPoint::OnRegistrationFailed (const PString & _aor,
 }
 
 
+void Opal::Sip::EndPoint::OnMWIReceived (const PString & party, OpalManager::MessageWaitingType /*type*/, const PString & info)
+{
+  std::string mwi = info;
+  std::transform (mwi.begin(), mwi.end(), mwi.begin(), ::tolower);
+  if (mwi == "no")
+    mwi = "0/0";
+
+  /* Signal */
+  Ekiga::Account *account = account_core.find_account (party);
+  if (account)
+    runtime.run_in_main (sigc::bind (account->mwi_event.make_slot (), info));
+}
+
+
 bool Opal::Sip::EndPoint::OnIncomingConnection (OpalConnection &connection,
                                                 unsigned options,
                                                 OpalConnection::StringOptions * stroptions)
@@ -953,7 +980,7 @@ bool Opal::Sip::EndPoint::OnIncomingConnection (OpalConnection &connection,
 }
 
 
-void Opal::Sip::EndPoint::OnReceivedMESSAGE (G_GNUC_UNUSED OpalTransport & transport,
+bool Opal::Sip::EndPoint::OnReceivedMESSAGE (OpalTransport & transport,
                                              SIP_PDU & pdu)
 {
   PString *last = NULL;
@@ -975,14 +1002,15 @@ void Opal::Sip::EndPoint::OnReceivedMESSAGE (G_GNUC_UNUSED OpalTransport & trans
     msgData.SetAt (SIPURL (from).AsString (), val);
 
     SIPURL uri = from;
-    uri.Sanitise (SIPURL::FromURI);
+    uri.Sanitise (SIPURL::RequestURI);
     std::string display_name = (const char *) uri.GetDisplayName ();
     std::string message_uri = (const char *) uri.AsString ();
     std::string _message = (const char *) pdu.GetEntityBody ();
 
-
     runtime.run_in_main (sigc::bind (sigc::ptr_fun (push_message_in_main), dialect, message_uri, display_name, _message));
   }
+
+  return SIPEndPoint::OnReceivedMESSAGE (transport, pdu);
 }
 
 
@@ -1027,7 +1055,7 @@ SIPURL Opal::Sip::EndPoint::GetRegisteredPartyName (const SIPURL & host)
       Ekiga::Account *account = account_core.find_account ("ekiga.net");
 
       if (account)
-        return SIPURL ("\"" + GetDefaultDisplayName () + "\" <" + account->get_aor () + ">");
+        return SIPURL ("\"" + GetDefaultDisplayName () + "\" <" + PString(account->get_aor ()) + ">");
     }
   }
 
@@ -1086,7 +1114,7 @@ Opal::Sip::EndPoint::OnPresenceInfoReceived (const PString & user,
    * TODO
    * Wouldn't it be convenient to emit the signal and have the presence core listen to it ?
    */
-  runtime.run_in_main (sigc::bind (sigc::ptr_fun (presence_status_in_main), &presence_core, _uri, presence, status));
+  runtime.run_in_main (sigc::bind (sigc::ptr_fun (presence_status_in_main), this, _uri, presence, status));
 }
 
 
@@ -1095,17 +1123,18 @@ void Opal::Sip::EndPoint::on_dial (std::string uri)
   manager.dial (uri);
 }
 
+
 void Opal::Sip::EndPoint::on_message (std::string uri,
                                       std::string name)
 {
   dialect->start_chat_with (uri, name);
 }
 
-void Opal::Sip::EndPoint::on_forward (std::string uri)
-{
-  PStringList connections = GetAllConnections ();
-  /* FIXME : we don't handle several connections here */
-  PSafePtr<OpalConnection> connection = GetConnectionWithLock (connections[0]);
 
-  connection->ForwardCall (uri);
+void Opal::Sip::EndPoint::on_transfer (std::string uri)
+{
+  /* FIXME : we don't handle several calls here */
+  for (PSafePtr<OpalConnection> connection(connectionsActive, PSafeReference); connection != NULL; ++connection)
+    if (!PIsDescendant(&(*connection), OpalPCSSConnection))
+      connection->TransferConnection (uri);
 }
